<?php
/**
* Modelo Rules para el Componente Securitycheckpro
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Chequeamos si el archivo est includo en Joomla!
defined('_JEXEC') or die();
jimport('joomla.application.component.model');
jimport('joomla.filesystem.file');


/**
* Modelo Securitycheck
*/
class SecuritycheckprosModelRules extends JModelLegacy
{

/** Objeto Pagination * @var object */
var $_pagination = null;

/** @var array Group list */
private $groups = array();

/** @var int Total number of files of Pagination */
var $total = 0;

function __construct()
{
	parent::__construct();
	
	
	$mainframe = JFactory::getApplication();
	
	// Obtenemos las variables de paginacin de la peticin
	$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
	$limitstart = JRequest::getVar('limitstart', 0, '', 'int');

	// En el caso de que los lmites hayan cambiado, los volvemos a ajustar
	$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

	$this->setState('limit', $limit);
	$this->setState('limitstart', $limitstart);
}

protected function populateState()
{
	// Inicializamos las variables
	$app		= JFactory::getApplication();
	
	$search = $app->getUserStateFromRequest('filter.acl_search', 'filter_acl_search');
	$this->setState('filter.acl_search', $search);
	
	parent::populateState();
}

/*  Funcin para la paginacin */
function getPagination()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_pagination)) {
	jimport('joomla.html.pagination');
$this->_pagination = new JPagination($this->total, $this->getState('limitstart'), $this->getState('limit') );
}
return $this->_pagination;
}

/* Funcin para cargar los grupos del sistema */
function load()
{

	// Creamos un nuevo objeto query
	$db = $this->getDbo();
	$query = $db->getQuery(true);

	// Obtenemos los grupos de Joomla
	$query_groups = 'SELECT * FROM ' . $db->quoteName('#__usergroups');
	$db->setQuery($query_groups);
	$joomla_groups = $db->loadObjectList();
	
	// Actualizamos el nmero total de grupos
	$this->total = count($joomla_groups);
	
	// Timestamp
	$date = JFactory::getDate();
	
	// Comprobamos si hay que aadir algn grupo a la tabla '#__securitycheckpro_rules'
	foreach ($joomla_groups as $element){
		$array_val = get_object_vars($element);
		$group_id = (int) $array_val["id"];
		$title = $array_val["title"];
		$rules_applied = (int) 1;
				
		$query_rules = 'SELECT * FROM ' . $db->quoteName("#__securitycheckpro_rules") . ' WHERE  group_id = ' . $group_id;
		
		$db->setQuery( $query_rules );
		$element_exists = $db->loadObjectList();
		
		// Al grupo 'Super Users' nunca se le deberan aplicar las reglas.
		if ( $title == 'Super Users' ) {
			$rules_applied = 0;
		}
		
		// Si no existe, lo aadimos
		if ( empty($element_exists) ) {
			$valor = (object) array(
						'group_id' => $group_id,
						'rules_applied' => $rules_applied,
						'last_change' => $date->format("Y-m-d H:i:s"),
						);
			$insert_result = $db->insertObject('#__securitycheckpro_rules', $valor, 'id');
		}
	}
	
	// Volvemos a construir la consulta
	$query->select('a.*');
	$query->from($db->quoteName('#__usergroups') . ' AS a');
		
	// Aadimos los niveles de cada grupo...
	$query->select('COUNT(DISTINCT c2.id) AS level')
		->join('LEFT OUTER', $db->quoteName('#__usergroups') . ' AS c2 ON a.lft > c2.lft AND a.rgt < c2.rgt')
		->group('a.id, a.lft, a.rgt, a.parent_id, a.title, b.rules_applied, b.last_change');
	
	// ... y si las reglas han de aplicarse
	$query->select('b.*')
		->join('LEFT OUTER', $db->quoteName('#__securitycheckpro_rules') . ' AS b ON a.id = b.group_id');
		
	// Filtramos los comentarios de las bsquedas si existen
	$search = $this->getState('filter.acl_search');
	if (!empty($search)) {
		if (stripos($search, 'id:') === 0) {
			$query->where('a.id = ' . (int) substr($search, 3));
		}
		else {
			$search = $db->quote('%' . $db->escape($search, true) . '%');
			$query->where('a.title LIKE ' . $search);
		}
	}
	
	// Ordenamos la consulta
		$query->order($db->escape($this->getState('list.ordering', 'a.lft')) . ' ' . $db->escape($this->getState('list.direction', 'ASC')));

	$db->setQuery($query);
	$groups = $db->loadObjectList();
		
	/* Obtenemos el nmero de registros del array que hemos de mostrar. Si el lmite superior es '0', entonces devolvemos todo el array */
	$upper_limit = $this->getState('limitstart');
	$lower_limit = $this->getState('limit');
	
	/* Devolvemos slo el contenido delimitado por la paginacin */
	$groups = array_splice($groups, $upper_limit, $lower_limit);
	
	return $groups;
}


/*  Funcin que cambia el estado de los grupos pasados como argumento. A dichos grupos se les aplicarn las reglas del Firewall. */
function apply_rules()
{
	$resultado = true;
	
	$uids = JRequest::getVar('cid', 0, '', 'array');
	
	// Timestamp
	$formatted_date = JFactory::getDate()->format("Y-m-d H:i:s");
	
	JArrayHelper::toInteger($uids, array());
		
	$db = $this->getDbo();
	foreach($uids as $uid) {
		$sql = "UPDATE `#__securitycheckpro_rules` SET rules_applied=1,last_change='" .$formatted_date ."' WHERE group_id='{$uid}'";
		$db->setQuery($sql);
		$db->execute();
		
		if ($db->getErrorNum()) {
			$this->setError(JText::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $db->getErrorNum(), $db->getErrorMsg()));
			$resultado = false;
			break(1);
		}
		
	}

	
return $resultado;
}

/*  Funcin que cambia el estado de los grupos pasados como argumento. A dichos grupos NO se les aplicarn las reglas del Firewall. */
function not_apply_rules()
{
	$resultado = true;
	
	$uids = JRequest::getVar('cid', 0, '', 'array');
	
	// Timestamp
	$formatted_date = JFactory::getDate()->format("Y-m-d H:i:s");
	
	JArrayHelper::toInteger($uids, array());
		
	$db = $this->getDbo();
	foreach($uids as $uid) {
		$sql = "UPDATE `#__securitycheckpro_rules` SET rules_applied=0,last_change='" .$formatted_date ."' WHERE group_id='{$uid}'";
		$db->setQuery($sql);
		$db->execute();
		
		if ($db->getErrorNum()) {
			$this->setError(JText::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $db->getErrorNum(), $db->getErrorMsg()));
			$resultado = false;
			break(1);
		}
		
	}

	
return $resultado;
}

}