<?php
/**
* @ author Jose A. Luque
* @ Copyright (c) 2013 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Protect from unauthorized access
defined('_JEXEC') or die();
jimport('joomla.filesystem.folder');

class SecuritycheckProsModelJson extends SecuritycheckproModel
{

	const	STATUS_OK					= 200;	// Normal reply
	const	STATUS_NOT_AUTH				= 401;	// Invalid credentials
	const	STATUS_NOT_ALLOWED			= 403;	// Not enough privileges
	const	STATUS_NOT_FOUND			= 404;  // Requested resource not found
	const	STATUS_INVALID_METHOD		= 405;	// Unknown JSON method
	const	STATUS_ERROR				= 500;	// An error occurred
	const	STATUS_NOT_IMPLEMENTED		= 501;	// Not implemented feature
	const	STATUS_NOT_AVAILABLE		= 503;	// Remote service not activated

	const	CIPHER_RAW			= 1;	// Data in plain-text JSON
	const	CIPHER_AESCBC128		= 2;	// Data in AES-128 standard (CBC) mode encrypted JSON
	const	CIPHER_AESCBC256		= 3;	// Data in AES-256 standard (CBC) mode encrypted JSON

	private	$json_errors = array(
		'JSON_ERROR_NONE' => 'No error has occurred (probably emtpy data passed)',
		'JSON_ERROR_DEPTH' => 'The maximum stack depth has been exceeded',
		'JSON_ERROR_CTRL_CHAR' => 'Control character error, possibly incorrectly encoded',
		'JSON_ERROR_SYNTAX' => 'Syntax error'
	);
	
	// Inicializamos las variables
	private	$status = 200;  // Estado de la peticin
	private $cipher = 2;	// Mtodo usado para cifrar los datos
	private $clear_data = '';		// Datos enviados en la peticin del cliente (ya en claro)
	public $data = '';		// Datos devueltos al cliente
	private $password = null;
	private $method_name = null;
	private $log_buffer = '******* Start of file ******* </br>';	// Buffer para almacenar el continido del fichero de logs
	private $createfolder = false;	// Se ha creado el directorio para guardar los resultados?
	private $remote_site = '';
	private $same_branch = true;	// Pertenecen los dos sitios a la misma versin de Joomla?
	private $stored_filename = '';	// Fichero remoto descargado
	private $database_name = '';	// Nombre del fichero .out
	private $maintain_db_structure = 0;	// Indica si hemos de mantener la estructura (establecida en configuration.php) del sitio local
	private $database_prefix = null;	// Prefijo de la BBDD local, necesaria si hemos de mantener la estructura de la BBDD local
	private $remote_database_prefix = null;	// Prefijo de la BBDD remota, necesaria si hemos de mantener la estructura de la BBDD local
	private $delete_existing_db = 0;	// Indica si hemos de borrar la BBDD local (aplicable slo si no hemos de mantener la estructura del sitio)
	private $cipher_file = 0;	// Indica si el fichero remoto est cifrado
	private $backupinfo = array('product'=> '', 'latest'=>'', 'latest_status'=>	'', 'latest_type'=>'');
	private $update_database_plugin_needs_update = false;   // Indica si el plugin 'Update Database' necesita actualizarse
	private $info = null;  // Contendr informacin sobre el sistema: versin de php, mysql y servidor
		
	/* Funcin que realiza una determinada funcin segn los parmetros especificados en la variable pasada como argumento */
	public function execute($json)
	{
				
		// Comprobamos si el frontend est habilitado
		$config = $this->Config('controlcenter');
		
		if ( is_null($config) ) {
			$this->data = "Can't get configuration";
			$this->status = self::STATUS_ERROR;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();
		}
		
		if ( !array_key_exists('control_center_enabled', $config) ) {
			$enabled = false;
		} else {
			$enabled = $config['control_center_enabled'];
		}
		
		if ( array_key_exists('secret_key', $config) ) {
			$this->password = $config['secret_key'];
		} else {
			$this->data = 'Remote password not configured';
			$this->status = self::STATUS_NOT_AUTH;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();
		}
		
		// Si el frontend no est habilitado, devolvemos un error 503
		if(!$enabled)
		{
			$this->data = 'Access denied';
			$this->status = self::STATUS_NOT_AVAILABLE;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();
		}
		
		$json_trimmed = rtrim($json, chr(0));

		// Comprobamos que el string JSON es vlido y que tiene al menos 12 caracteres (longitud mnima de un mensaje vlido)
		if ((strlen($json_trimmed) < 12) || (substr($json_trimmed, 0, 1) != '{') || (substr($json_trimmed, -1) != '}')) {
			// El string JSON no es vlido, devolvemos un error
			$this->data = 'JSON decoding error';
			$this->status = self::STATUS_ERROR;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();	
		} else {
			// Decodificamos la peticin
			$request = json_decode($json, true);
		}
		
		if(is_null($request)) {
			// Si no podemos decodificar la peticin JSON, devolvemos un error
			$this->data = 'JSON decoding error';
			$this->status = self::STATUS_ERROR;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();			
		} 
		
		// Decodificamos el 'body' de la peticin
		if( isset($request['cipher']) && isset($request['body']) ) {
			switch( $request['cipher'] ) {
				case self::CIPHER_RAW:
					if ( ($request['body']['task'] == "getStatus") || ($request['body']['task'] == "checkVuln") || ($request['body']['task'] == "checkLogs") || ($request['body']['task'] == "checkPermissions") || ($request['body']['task'] == "checkIntegrity") || ($request['body']['task'] == "deleteBlocked") || ($request['body']['task'] == "checkmalware") || ($request['body']['task'] == "UpdateExtension") || ($request['body']['task'] == "Backup") ) {
						/* Los resultados de todas las tareas se devuelven cifrados; si recibimos una peticin para devolverlos sin cifrar, la rechazamos
							porque ser fraudulenta */
						$this->data = 'Go away, hacker!';
						$this->status = self::STATUS_NOT_ALLOWED;
						$this->cipher = self::CIPHER_RAW;
						return $this->sendResponse();
					}
					break;
					
				case self::CIPHER_AESCBC128:
					if ( !is_null($request['body']['data']) ) {
						//$this->clear_data = $this->mc_decrypt_128($request->body->data, $this->password);		
					}
					break;

				case self::CIPHER_AESCBC256:
					if ( !is_null($request['body']['data']) ) {
						//$this->clear_data = $this->mc_decrypt_256($request->body->data, $this->password);
					}					
					break;
			}
			
				
			$this->cipher = $request['cipher'];
			switch( $request['body']['task'] ) {
				case "getStatus":
					$this->getStatus();
					break;
					
				case "checkVuln":
					$this->checkVuln();
					break;
					
				case "checkLogs":
					$this->checkLogs();
					break;
					
				case "checkPermissions":
					$this->checkPermissions();
					break;
					
				case "checkIntegrity":
					$this->checkIntegrity();
					break;
					
				case "deleteBlocked":
					$this->deleteBlocked();
					break;
					
				case "LatestReleaseInfo":
					$this->LatestReleaseInfo();
					break;
					
				case "checkmalware":
					$this->checkMalware();
					break;
					
				case "UpdateComponent":
					$this->UpdateComponent();
					break;
					
				case "UpdateExtension":
					$this->UpdateExtension($request['body']['data']);
					break;
					
				case "Backup":
					$this->Backup($request['body']['data']);
					break;
					
				case "Uploadinstall":
					$this->Upload_install($request['body']['data']);
					break;
					
				case self::CIPHER_AESCBC256:
									
					break;
				default:
					$this->data = 'Method not configured';
					$this->status = self::STATUS_NOT_FOUND;
					$this->cipher = self::CIPHER_RAW;
					return $this->sendResponse();
			}
			return $this->sendResponse();		
		}
	}
	
	/* Funcin que empaqueta una respuesta en formato JSON codificado, cifrando los datos si es necesario */
	private function sendResponse()
	{
		// Inicializamos la respuesta
		$response = array(
			'cipher'	=> $this->cipher,
			'body'		=> array(
				'status'		=> $this->status,
				'data'			=> null
			)
		);
		
		// Codificamos los datos enviados en formato JSON
		$data = json_encode($this->data);
		
		// Ciframos o no los datos segn el mtodo establecido en la peticin
		switch($this->cipher)
		{
			case self::CIPHER_RAW:
				break;

			case self::CIPHER_AESCBC128:
				$data = $this->encrypt($data, $this->password);				
				break;

			case self::CIPHER_AESCBC256:
				//$data = $this->mc_encrypt_256($data, $this->password);
				break;
		}

		// Guardamos los datos...
		$response['body']['data'] = $data;

		// ... y los devolvemos al cliente
		return '###' . json_encode($response) . '###';		
	}
	
	/* Extraemos los parmetros del componente */
	private function Config($key_name)
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		$query 
			->select($db->quoteName('storage_value'))
			->from($db->quoteName('#__securitycheckpro_storage'))
			->where($db->quoteName('storage_key').' = '.$db->quote($key_name));
		$db->setQuery($query);
		$res = $db->loadResult();
		$res = json_decode($res, true);
			
		return $res;
	}

	/* Funcin que devuelve el estado de la extensin remota  */
	public function getStatus($opcion=true) {
	
		// Inicializamos las variables
		$extension_updates = null;
		
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('cpanel', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		JLoader::import('filemanager', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		JLoader::import('databaseupdates', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'helpers');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$cpanel_model = JModelLegacy::getInstance( 'cpanel', 'SecuritycheckprosModel');
			$filemanager_model = JModelLegacy::getInstance( 'filemanager', 'SecuritycheckprosModel');
			$update_model = JModelLegacy::getInstance( 'databaseupdates', 'SecuritycheckprosModel');
		} else {
			$cpanel_model = JModel::getInstance( 'cpanel', 'SecuritycheckprosModel');
			$filemanager_model = JModel::getInstance( 'filemanager', 'SecuritycheckprosModel');
			$update_model = JModel::getInstance( 'databaseupdates', 'SecuritycheckprosModel');
		}
		
		if ( (empty($cpanel_model)) || (empty($filemanager_model)) || (empty($update_model)) ) {
			return;
		}
		
		// Comprobamos el estado del plugin Update Database
		$update_database_plugin_installed = $update_model-> PluginStatus(4);
		$update_database_plugin_version = $update_model->get_database_version();
		$update_database_plugin_last_check = $update_model->last_check();
				
		// Check for vulnerable components
		$cpanel_model->buscarQuickIcons();
		
		// Vulnerable components
		$db = JFactory::getDBO();
		$query = 'SELECT COUNT(*) FROM #__securitycheckpro WHERE Vulnerable="Si"';
		$db->setQuery( $query );
		$db->execute();	
		$vuln_extensions = $db->loadResult();
		
		// Check for unread logs
		(int) $logs_pending = $cpanel_model->LogsPending();
		
		// Get files with incorrect permissions from database
		$files_with_incorrect_permissions = $filemanager_model->loadStack("filemanager_resume","files_with_incorrect_permissions");
		
		// If permissions task has not been launched, we set a '0' value.
		if ( is_null($files_with_incorrect_permissions) ) {
			$files_with_incorrect_permissions = 0;
		}
		
		// FileManager last check
		$last_check = $filemanager_model->loadStack("filemanager_resume","last_check");
		
		// Get files with incorrect permissions from database
		$files_with_bad_integrity = $filemanager_model->loadStack("fileintegrity_resume","files_with_bad_integrity");
		
		// If permissions task has not been launched, whe set a '0' value.
		if ( is_null($files_with_bad_integrity) ) {
			$files_with_bad_integrity = 0;
		}
		
		// FileIntegrity last check
		$last_check_integrity = $filemanager_model->loadStack("fileintegrity_resume","last_check_integrity");
		
		// Malwarescan last check
		$last_check_malwarescan = $filemanager_model->loadStack("malwarescan_resume","last_check_malwarescan");
		
		// Get suspicious files
		$suspicious_files = $filemanager_model->loadStack("malwarescan_resume","suspicious_files");
		
		// ltima optimizacin bbdd
		$last_check_database_optimization = $this->get_campo_filemanager('last_check_database');
		
		// If malwarescan has not been launched, we set a '0' value.
		if ( is_null($suspicious_files) ) {
			$suspicious_files = 0;
		}
		
		// Comprobamos el estado del backup
		$this->getBackupInfo();
		
		// Verificamos si el cliente est actualizado
		require_once JPATH_ROOT.'/administrator/components/com_securitycheckpro/liveupdate/liveupdate.php';
		$liveupdatemodel = new LiveUpdate();
		$updateInformation = $liveupdatemodel->getUpdateInformation(1);
		
		// Verificamos si el core est actualizado (obviando la cach) 
		require_once JPATH_ROOT.'/administrator/components/com_joomlaupdate/models/default.php';
		$updatemodel = new JoomlaupdateModelDefault();
		$updatemodel->refreshUpdates(true);
		$coreInformation = $updatemodel->getUpdateInformation();
						
		// Si el plugin 'Update Batabase' est instalado, comprobamos si est actualizado
		if ( $update_database_plugin_installed ) {
			$this->update_database_plugin_needs_update = $this->checkforUpdate();
		} else {
			$this->update_database_plugin_needs_update = null;
		}
		
		// Aadimos la informacin del sistema
		$this->getInfo();
		
		// Aadimos la informacin sobre las extensiones no actualizadas. Esta opcin no es necesaria cuando escogemos la opcin 'System Info'
		if ( $opcion ) {
			$extension_updates = $this->getNotUpdatedExtensions();
		}	
		
		$this->data = array(
			'vuln_extensions'		=> $vuln_extensions,
			'logs_pending'	=> $logs_pending,
			'files_with_incorrect_permissions'		=> $files_with_incorrect_permissions,
			'last_check' => $last_check,
			'files_with_bad_integrity'		=> $files_with_bad_integrity,
			'last_check_integrity' => $last_check_integrity,
			'installed_version'	=> $updateInformation->extInfo->version,
			'hasUpdates'	=> $updateInformation->hasUpdates,
			'coreinstalled'	=>	$coreInformation['installed'],
			'corelatest'	=>	$coreInformation['latest'],
			'last_check_malwarescan' => $last_check_malwarescan,
			'suspicious_files'		=> $suspicious_files,
			'update_database_plugin_installed'	=>	$update_database_plugin_installed,
			'update_database_plugin_version'	=>	$update_database_plugin_version,
			'update_database_plugin_last_check'	=>	$update_database_plugin_last_check,
			'update_database_plugin_needs_update'	=>	$this->update_database_plugin_needs_update,
			'backup_info_product'	=>	$this->backupinfo['product'],
			'backup_info_latest'	=>	$this->backupinfo['latest'],
			'backup_info_latest_status'	=>	$this->backupinfo['latest_status'],
			'backup_info_latest_type'	=>	$this->backupinfo['latest_type'],
			'php_version'	=>	$this->info['phpversion'],
			'database_version'	=>	$this->info['dbversion'],
			'web_server'	=>	$this->info['server'],
			'extension_updates'	=>	$extension_updates,
			'last_check_database_optimization'	=> $last_check_database_optimization
		);		
				
	}
	
	/* Funcin que comprueba si existen extensiones vulnerables  */
	private function checkVuln() {
	
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('securitycheckpros', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		JLoader::import('databaseupdates', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'helpers');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$securitycheckpros_model = JModelLegacy::getInstance( 'securitycheckpros', 'SecuritycheckprosModel');
			$update_model = JModelLegacy::getInstance( 'databaseupdates', 'SecuritycheckprosModel');
		} else {
			$securitycheckpros_model = JModel::getInstance( 'securitycheckpros', 'SecuritycheckprosModel');
			$update_model = JModel::getInstance( 'databaseupdates', 'SecuritycheckprosModel');
		}
		
		// Comprobamos si existen nuevas actualizaciones
		$result = $update_model->tarea_comprobacion();
		
		// Comprobamos el estado del plugin Update Database
		$update_database_plugin_installed = $update_model-> PluginStatus(4);
		$update_database_plugin_version = $update_model->get_database_version();
		$update_database_plugin_last_check = $update_model->last_check();

		// Hacemos una nueva comprobacin de extensiones vulnerables
		$securitycheckpros_model->chequear_vulnerabilidades();
		
		// Vulnerable components
		$db = JFactory::getDBO();
		$query = 'SELECT COUNT(*) FROM #__securitycheckpro WHERE Vulnerable="Si"';
		$db->setQuery( $query );
		$db->execute();	
		$vuln_extensions = $db->loadResult();
		
		$this->data = array(
			'vuln_extensions'		=> $vuln_extensions,
			'update_database_plugin_installed'	=>	$update_database_plugin_installed,
			'update_database_plugin_version'	=>	$update_database_plugin_version,
			'update_database_plugin_last_check'	=>	$update_database_plugin_last_check
		);
	}
	
	/* Funcin que comprueba si existen logs por leer  */
	private function checkLogs() {
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('cpanel', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$cpanel_model = JModelLegacy::getInstance( 'cpanel', 'SecuritycheckprosModel');			
		} else {
			$cpanel_model = JModel::getInstance( 'cpanel', 'SecuritycheckprosModel');			
		}
		
		// Check for unread logs
		(int) $logs_pending = $cpanel_model->LogsPending();
		
		$this->data = array(
			'logs_pending'	=> $logs_pending			
		);
		
	}
	
	/* Funcin que lanza un chequeo de permisos  */
	private function checkPermissions() {
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('filemanager', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$filemanager_model = JModelLegacy::getInstance( 'filemanager', 'SecuritycheckprosModel');
		} else {
			$filemanager_model = JModel::getInstance( 'filemanager', 'SecuritycheckprosModel');
		}
		
		$filemanager_model->set_campo_filemanager('files_scanned',0);
		$filemanager_model->set_campo_filemanager('last_check',date('Y-m-d H:i:s'));
		$filemanager_model->set_campo_filemanager('estado','IN_PROGRESS');
		$filemanager_model->scan("permissions");
		
		// Get files with incorrect permissions from database
		$files_with_incorrect_permissions = $filemanager_model->loadStack("filemanager_resume","files_with_incorrect_permissions");
		
		// If permissions task has not been launched, we set a '0' value.
		if ( is_null($files_with_incorrect_permissions) ) {
			$files_with_incorrect_permissions = 0;
		}
		
		// FileManager last check
		$last_check = $filemanager_model->loadStack("filemanager_resume","last_check");
		
		$this->data = array(
			'files_with_incorrect_permissions'		=> $files_with_incorrect_permissions,
			'last_check' => $last_check
		);
	
	}
	
	/* Funcin que lanza un chequeo de integridad  */
	private function checkIntegrity() {
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('filemanager', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$filemanager_model = JModelLegacy::getInstance( 'filemanager', 'SecuritycheckprosModel');
		} else {
			$filemanager_model = JModel::getInstance( 'filemanager', 'SecuritycheckprosModel');
		}
		
		$filemanager_model->set_campo_filemanager('files_scanned_integrity',0);
		$filemanager_model->set_campo_filemanager('last_check_integrity',date('Y-m-d H:i:s'));
		$filemanager_model->set_campo_filemanager('estado_integrity','IN_PROGRESS');
		$filemanager_model->scan("integrity");
		
		// Get files with incorrect permissions from database
		$files_with_bad_integrity = $filemanager_model->loadStack("fileintegrity_resume","files_with_bad_integrity");
		
		// If permissions task has not been launched, we set a '0' value.
		if ( is_null($files_with_bad_integrity) ) {
			$files_with_bad_integrity = 0;
		}
		
		// FileIntegrity last check
		$last_check_integrity = $filemanager_model->loadStack("fileintegrity_resume","last_check_integrity");
		
		$this->data = array(
			'files_with_bad_integrity'		=> $files_with_bad_integrity,
			'last_check_integrity' => $last_check_integrity
		);
	
	}
	
	/* Borra los logs pertenecientes a intentos de acceso bloqueados */
	private function deleteBlocked() {
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('cpanel', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$cpanel_model = JModelLegacy::getInstance( 'cpanel', 'SecuritycheckprosModel');			
		} else {
			$cpanel_model = JModel::getInstance( 'cpanel', 'SecuritycheckprosModel');			
		}
	
		// Vulnerable components
		$db = JFactory::getDBO();
		$query = 'DELETE FROM #__securitycheckpro_logs WHERE ( `type` = "IP_BLOCKED" OR `type` = "IP_BLOCKED_DINAMIC" )';
		$db->setQuery( $query );
		$db->execute();	
				
		// Check for unread logs
		(int) $logs_pending = $cpanel_model->LogsPending();
		
		$this->data = array(
			'logs_pending'	=> $logs_pending			
		);
	}
	
	private function Update()
	{
		// Download 
		require_once JPATH_ROOT.'/administrator/components/com_securitycheckpro/liveupdate/liveupdate.php';
		require_once JPATH_ROOT.'/administrator/components/com_securitycheckpro/liveupdate/classes/model.php';

		// Do we need to update?
		$updateInformation = LiveUpdate::getUpdateInformation();
		if(!$updateInformation->hasUpdates) {
			return (object)array(
				'download'	=> 0
			);
		}

		$model = new LiveupdateModel();
		$ret = $model->download();

		$session = JFactory::getSession();
		$target		= $session->get('target', '', 'liveupdate');
		$tempdir	= $session->get('tempdir', '', 'liveupdate');

		if(!$ret) {
			// An error ocurred :(
			$this->data = 'Could not download the update package';
			$this->status = self::STATUS_ERROR;
			$this->cipher = self::CIPHER_RAW;
			return $this->sendResponse();			
		} else {
			// Extract
			$ret = $model->extract();

			JLoader::import('joomla.filesystem.file');
			JFile::delete($target);
			
			if(!$ret) {
				// An error ocurred :(
				$this->data = 'Could not extract the update package';
				$this->status = self::STATUS_ERROR;
				$this->cipher = self::CIPHER_RAW;
				return $this->sendResponse();
			} else {
				// Install
				$ret = $model->install();

				if(!$ret) {
					// An error ocurred :(
					$this->data = 'Could not install the update package';
					$this->status = self::STATUS_ERROR;
					$this->cipher = self::CIPHER_RAW;
					return $this->sendResponse();					
				} else {
					// Update cleanup
					$ret = $model->cleanup();

					JLoader::import('joomla.filesystem.file');
					JFile::delete($target);
					
					// Update product info
					$this->getStatus();
				}
			}
		}
	}
	
	/* Obtiene informacin de la ltima versin publicada */
	private function LatestReleaseInfo() {
		/* Preguntamos por la informacin de la ltima versin */
		require_once JPATH_ROOT.'/administrator/components/com_securitycheckpro/liveupdate/liveupdate.php';
		$updateInformation = LiveUpdate::getUpdateInformation(1);
		
	
		$this->data = array(
			'latest_version'	=> $updateInformation->version,
			'release_notes'	=> $updateInformation->releasenotes
		);
	
	}
	
	/* Obtiene informacin de los requisitos necesarios para clonar una web */
	private function CheckPrereq() {
		
		/* Inicializamos las variables */
		$server_type = 0;  // Sistema operativo 'Linux'
		$safe_mode = 0;
		$mysqldump = null;
		$tar = null;
		
		/* Chequeamos los requisitos */
		// Tipo de servidor
		$os = php_uname("s");
		if ( strstr($os,'Windows') ){
			$server_type = 1;
		} else if ( strstr($os,'Mac') ){
			$server_type = 2;
		}
		
		// 'Safe_mode'
		if ( ini_get('safe_mode') ) {
			$safe_mode = 1;
		}
			
		$this->data = array(
			'server_type'	=> $server_type,
			'safe_mode'	=> $safe_mode,
			'mysqldump'	=> $mysqldump,
			'tar'	=> $tar
		);		
		
	}
	
	/* Funcin que actualiza el Core de Joomla a la ltima versin disponible  */
	private function UpdateCore() {
	
		/* Cargamos el lenguaje del componente 'com_installer' */
		$lang = JFactory::getLanguage();
		$lang->load('com_installer',JPATH_ADMINISTRATOR);
		
		// Inicializamos la variable $result, que ser un array con el resultado y el mensaje devuelto en el proceso
		$result = array();
		
		// Cargamos las libreras necesarias
		require_once JPATH_ROOT.'/administrator/components/com_joomlaupdate/models/default.php';
		
		// Instanciamos el modelo de la librera anteriormente cargada		
		$model = new JoomlaupdateModelDefault();
				
		// Refrescamos la informacin de las actualizaciones ignorando la cach
		$model->refreshUpdates(true);
		
		// Extraemos la url de descarga
		$coreInformation = $model->getUpdateInformation();
		
		// Descargamos el archivo
		$file = $this->download_core($coreInformation['object']->downloadurl->_data);
		
		// Extract the downloaded package file
		$config   = JFactory::getConfig();
		$tmp_dest = $config->get('tmp_path');
				
		$zip = new ZipArchive;
		$res = $zip->open($tmp_dest . DIRECTORY_SEPARATOR .$file);
		if ($res === true) {
			$zip->extractTo(JPATH_SITE);
			$zip->close();			
		}
		
		// Create restoration file
		$this->createRestorationFile($file);
		
		$install_result = $this->finaliseUpgrade();
						
		if ( !$install_result ) {
			$msg = JText::_('COM_INSTALLER_MSG_UPDATE_ERROR');
			$result[0][1] = $msg;
			$result[0][0] = false;			
		} else {
			$result[0][1] = '';			
			$result[0][0] = true;
			// Clean the site
			JFile::delete($tmp_dest . DIRECTORY_SEPARATOR .$file);
			JFile::delete(JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR .'com_joomlaupdate' . DIRECTORY_SEPARATOR . 'restoration.php');
		}
		
		// Devolvemos el resultado
		return $result;		
	}
	
	
	/**
	 * Install an extension from either folder, url or upload.
	 *
	 * @return  boolean result of install
	 *
	 * @since   1.5
	 */
	public function install($url)
	{
		$this->setState('action', 'install');

		// Set FTP credentials, if given.
		JClientHelper::setCredentialsFromRequest('ftp');
		$app = JFactory::getApplication();

		// Load installer plugins for assistance if required:
		JPluginHelper::importPlugin('installer');
		$dispatcher = JEventDispatcher::getInstance();

		$package = null;

		// This event allows an input pre-treatment, a custom pre-packing or custom installation (e.g. from a JSONdescription)
		$results = $dispatcher->trigger('onInstallerBeforeInstallation', array($this, &$package));

		if (in_array(true, $results, true))
		{
			return true;
		}
		elseif (in_array(false, $results, true))
		{
			return false;
		}

		$installType = 'url';

		if ($package === null)
		{
			switch ($installType)
			{
				case 'folder':
					// Remember the 'Install from Directory' path.
					$app->getUserStateFromRequest($this->_context . '.install_directory', 'install_directory');
					$package = $this->_getPackageFromFolder();
					break;

				case 'upload':
					$package = $this->_getPackageFromUpload();
					break;

				case 'url':
					$package = $this->_getPackageFromUrl($url);
					break;

				default:
					$app->setUserState('com_installer.message', JText::_('COM_INSTALLER_NO_INSTALL_TYPE_FOUND'));

					return false;
					break;
			}
		}

		// This event allows a custom installation of the package or a customization of the package:
		$results = $dispatcher->trigger('onInstallerBeforeInstaller', array($this, &$package));

		if (in_array(true, $results, true))
		{
			return true;
		}
		elseif (in_array(false, $results, true))
		{
			if (in_array($installType, array('upload', 'url')))
			{
				JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);
			}

			return false;
		}

		// Was the package unpacked?
		if (!$package || !$package['type'])
		{
			if (in_array($installType, array('upload', 'url')))
			{
				JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);
			}


			$app->setUserState('com_installer.message', JText::_('COM_INSTALLER_UNABLE_TO_FIND_INSTALL_PACKAGE'));
			return false;
		}

		// Get an installer instance
		$installer = JInstaller::getInstance();

		// Install the package
		if (!$installer->install($package['dir']))
		{
			// There was an error installing the package
			$msg = JText::sprintf('COM_INSTALLER_INSTALL_ERROR', JText::_('COM_INSTALLER_TYPE_TYPE_' . strtoupper($package['type'])));
			$result = false;
		}
		else
		{
			// Package installed sucessfully
			$msg = JText::sprintf('COM_INSTALLER_INSTALL_SUCCESS', JText::_('COM_INSTALLER_TYPE_TYPE_' . strtoupper($package['type'])));
			$result = true;
		}

		// This event allows a custom a post-flight:
		$dispatcher->trigger('onInstallerAfterInstaller', array($this, &$package, $installer, &$result, &$msg));

		// Set some model state values
		$app	= JFactory::getApplication();
		$app->enqueueMessage($msg);
		$this->setState('name', $installer->get('name'));
		$this->setState('result', $result);
		$app->setUserState('com_installer.message', $installer->message);
		$app->setUserState('com_installer.extension_message', $installer->get('extension_message'));
		$app->setUserState('com_installer.redirect_url', $installer->get('redirect_url'));

		// Cleanup the install files
		if (!is_file($package['packagefile']))
		{
			$config = JFactory::getConfig();
			$package['packagefile'] = $config->get('tmp_path') . '/' . $package['packagefile'];
		}

		JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

		return $result;
	}
	
	
	/**
	 * Install an extension from a URL
	 *
	 * @return  Package details or false on failure
	 *
	 * @since   1.5
	 */
	protected function _getPackageFromUrl($url)
	{
		$input = JFactory::getApplication()->input;

		// Get the URL of the package to install
		//$url = $input->getString('install_url');

		// Did you give us a URL?
		if (!$url)
		{
			JError::raiseWarning('', JText::_('COM_INSTALLER_MSG_INSTALL_ENTER_A_URL'));
			return false;
		}

		// Handle updater XML file case:
		if (preg_match('/\.xml\s*$/', $url))
		{
			jimport('joomla.updater.update');
			$update = new JUpdate;
			$update->loadFromXML($url);
			$package_url = trim($update->get('downloadurl', false)->_data);
			if ($package_url)
			{
				$url = $package_url;
			}
			unset($update);
		}

		// Download the package at the URL given
		$p_file = JInstallerHelper::downloadPackage($url);

		// Was the package downloaded?
		if (!$p_file)
		{
			JError::raiseWarning('', JText::_('COM_INSTALLER_MSG_INSTALL_INVALID_URL'));
			return false;
		}

		$config   = JFactory::getConfig();
		$tmp_dest = $config->get('tmp_path');

		// Unpack the downloaded package file
		$package = JInstallerHelper::unpack($tmp_dest . '/' . $p_file, true);

		return $package;
	}
	
	/* Funcin que lanza un chequeo en busca de malware  */
	private function checkMalware() {
		// Import Securitycheckpros model
		JLoader::import('joomla.application.component.model');
		JLoader::import('filemanager', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
		if ( version_compare(JVERSION, '3.0', 'ge') ) {
			$filemanager_model = JModelLegacy::getInstance( 'filemanager', 'SecuritycheckprosModel');
		} else {
			$filemanager_model = JModel::getInstance( 'filemanager', 'SecuritycheckprosModel');
		}
		
		$filemanager_model->set_campo_filemanager('files_scanned_malwarescan',0);
		$filemanager_model->set_campo_filemanager('last_check_malwarescan',date('Y-m-d H:i:s'));
		$filemanager_model->set_campo_filemanager('estado_malwarescan','IN_PROGRESS');
		$filemanager_model->scan("malwarescan");
		
		// Get suspicious files
		$suspicious_files = $filemanager_model->loadStack("malwarescan_resume","suspicious_files");
		
		// If malwarescan task has not been launched, we set a '0' value.
		if ( is_null($suspicious_files) ) {
			$suspicious_files = 0;
		}
		
		// Malwarescan last check
		$last_check_malwarescan = $filemanager_model->loadStack("malwarescan_resume","last_check_malwarescan");
		
		$this->data = array(
			'suspicious_files'		=> $suspicious_files,
			'last_check_malwarescan' => $last_check_malwarescan
		);
	
	}
	
	/* Funcin que obtiene informacin del estado del backup  */
	private function getBackupInfo() {
		
		// Instanciamos la consulta
		$db = JFactory::getDBO();
		
		// Consultamos si Akeeba Backup est instalado
		$query = 'SELECT COUNT(*) FROM #__extensions WHERE element="com_akeeba"';
		$db->setQuery( $query );
		$db->execute();	
		$akeeba_installed = $db->loadResult();
		
		if ( $akeeba_installed == 1 ) {
			$this->backupinfo['product'] = 'Akeeba Backup';
			$this->AkeebaBackupInfo();
		} else {
			
			// Consultamos si Xcloner Backup and Restore est instalado
			$query = 'SELECT COUNT(*) FROM #__extensions WHERE element="com_xcloner-backupandrestore"';
			$db->setQuery( $query );
			$db->execute();	
			$xcloner_installed = $db->loadResult();
			
			if ( $xcloner_installed == 1 ) {
				$this->backupinfo['product'] = 'Xcloner - Backup and Restore';
				$this->XclonerbackupInfo();				
			} else {
			
				// Consultamos si Easy Joomla Backup est instalado
				$query = 'SELECT COUNT(*) FROM #__extensions WHERE element="com_easyjoomlabackup"';
				$db->setQuery( $query );
				$db->execute();	
				$ejb_installed = $db->loadResult();
				
				if ( $ejb_installed == 1 ) {
					$this->backupinfo['product'] = 'Easy Joomla Backup';
					$this->EjbInfo();				
				} 
			}
		}
		
	}
	
	/* Funcin que obtiene informacin del estado del ltimo backup creado por Akeeba Backup  */
	private function AkeebaBackupInfo() {
		
		// Instanciamos la consulta
		$db = JFactory::getDBO();
		$query = $db->getQuery(true)
			->select('MAX('.$db->qn('id').')')
			->from($db->qn('#__ak_stats'))
			->where($db->qn('origin') .' != '.$db->q('restorepoint'));
		$db->setQuery($query);
		$id = $db->loadResult();
		
		// Hay al menos un backup creado
		if ( !empty($id) ) {
			$query = $db->getQuery(true)
				->select(array('*'))
				->from($db->quoteName('#__ak_stats'))
				->where('id = '.$id);				
			$db->setQuery($query);
			$backup_statistics = $db->loadAssocList();			
						
			// Almacenamos el resultado
			$this->backupinfo['latest'] = $backup_statistics[0]['backupend'];
			$this->backupinfo['latest_status'] = $backup_statistics[0]['status'];
			$this->backupinfo['latest_type'] = $backup_statistics[0]['type'];
		}
	}
	
	/* Funcin que obtiene informacin del estado del ltimo backup creado por Xcloner - Backup and Restore  */
	private function XclonerbackupInfo() {
		
		// Incluimos el fichero de configuracin de la extensin
		include JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . "components" . DIRECTORY_SEPARATOR . "com_xcloner-backupandrestore" . DIRECTORY_SEPARATOR . "cloner.config.php";
		
		// Extraemos el directorio donde se encuentran almacenados los backups...
		$backup_dir = $_CONFIG['clonerPath'];
		
		// ... y buscamos dentro los ficheros existentes, ordenndolos por fecha
		$files_name = JFolder::files($backup_dir,'.',true,true);
		$files_name = array_combine($files_name, array_map("filemtime",$files_name));
		arsort($files_name);
		
		// El primer elemento del array ser el que se ha creado el ltimo. Formateamos la fecha para guardarlo en la BBDD.
		$latest_backup = date("Y-m-d H:i:s",filemtime(key($files_name)));
		
		// Almacenamos el resultado
		$this->backupinfo['latest'] = $latest_backup;
		$this->backupinfo['latest_status'] = 'complete';
		
	}
	
	/* Funcin que obtiene informacin del estado del ltimo backup creado por Easy Joomla Backup  */
	private function EjbInfo() {
		
		// Instanciamos la consulta
		$db = JFactory::getDBO();
		$query = $db->getQuery(true)
			->select('MAX('.$db->qn('id').')')
			->from($db->qn('#__easyjoomlabackup'));
		$db->setQuery($query);
		$id = $db->loadResult();
		
		// Hay al menos un backup creado
		if ( !empty($id) ) {
			$query = $db->getQuery(true)
				->select(array('*'))
				->from($db->quoteName('#__easyjoomlabackup'))
				->where('id = '.$id);				
			$db->setQuery($query);
			$backup_statistics = $db->loadAssocList();			
						
			// Almacenamos el resultado
			$this->backupinfo['latest'] = $backup_statistics[0]['date'];
			$this->backupinfo['latest_status'] = 'complete';
			$this->backupinfo['latest_type'] = $backup_statistics[0]['type'];
		}
		
	}
	
	/* Funcin que indica si el plugin 'Update Database' est actualizado */
	private function checkforUpdate() {
	
		//Inicializmaos las variables
		$needs_update = false;
		
		$db = JFactory::getDBO();
		
		// Extraemos el id de la extensin..
		$query = 'SELECT extension_id FROM #__extensions WHERE name="System - Securitycheck Pro Update Database"';
		$db->setQuery( $query );
		$db->execute();
		(int) $extension_id = $db->loadResult();
		
		// ... y hacemos una consulta a la tabla 'updates' para ver si el 'extension_id' figura como actualizable
		if ( !empty($extension_id) ) {
			$query = "SELECT COUNT(*) FROM #__updates WHERE extension_id={$extension_id}";
			$db->setQuery( $query );
			$db->execute();
			$result = $db->loadResult();
			
			if ( $result == '1' ) {
				$needs_update = true;
			}
			
		}
		
		// Devolvemos el resultado
		return $needs_update;		
		
	}
	
	/* Funcin que actualiza el plugin 'Update Database' */
	private function UpdateComponent() {
	
		// Inicializamos las variables
		$needs_update = 1;
		jimport( 'joomla.updater.update' );
		
		$db = JFactory::getDBO();
		
		// Extraemos el id de la extensin..
		$query = 'SELECT extension_id FROM #__extensions WHERE name="System - Securitycheck Pro Update Database"';
		$db->setQuery( $query );
		$db->execute();
		(int) $extension_id = $db->loadResult();
		
		$query = "SELECT detailsurl FROM #__updates WHERE extension_id={$extension_id}";
		//$query = 'SELECT extension_id FROM #__extensions WHERE name="System - Securitycheck Pro Update Database"';
		$db->setQuery( $query );
		$db->execute();
		$detailsurl = $db->loadResult();
		
		// Instanciamos el objeto JUpdate y cargamos los detalles de la actualizacin
		$update = new JUpdate();
		$update->loadFromXML($detailsurl);
		
		// Le pasamos a la funcin de actualizacin el objeto con los detalles de la actualizacin
		$result= $this->install_update($update);
		
		// Si la actualizacin ha tenido xito, actualizamos la variable 'needs_update', que indica si el plugin necesita actualizarse.
		if ( $result ) {
			$needs_update = 0;
		}
		
		// Devolvemos el resultado
		$this->data = array(
			'update_plugin_needs_update' => $needs_update
		);
	}
	
	/* Funcin para actualizar los componentes. Extrada del core de Joomla (administrator/components/com_installer/models/update.php) */
	private function install_update($update) {
		/* Cargamos el lenguaje del componente 'com_installer' */
		$lang = JFactory::getLanguage();
		$lang->load('com_installer',JPATH_ADMINISTRATOR);
	
		// Inicializamos la variable $result, que ser un array con el resultado y el mensaje devuelto en el proceso
		$result = array();
		$app = JFactory::getApplication();
		
		if (isset($update->get('downloadurl')->_data)) {
			$url = trim($update->downloadurl->_data);
		} else {
			$result[0][1] = JError::raiseWarning('', JText::_('COM_INSTALLER_INVALID_EXTENSION_UPDATE'));
			$result[0][0] = false;
		}

		$p_file = JInstallerHelper::downloadPackage($url);

		// Was the package downloaded?
		if (!$p_file) {
			$result[0][1] = JError::raiseWarning('', JText::sprintf('COM_INSTALLER_PACKAGE_DOWNLOAD_FAILED', $url));
			$result[0][0] = false;
		}

		$config		= JFactory::getConfig();
		$tmp_dest	= $config->get('tmp_path');

		// Unpack the downloaded package file
		$package	= JInstallerHelper::unpack($tmp_dest . '/' . $p_file);

		// Get an installer instance
		$installer	= JInstaller::getInstance();
		$update->set('type', $package['type']);

		// Install the package
		if (!$installer->update($package['dir'])) {
			// There was an error updating the package
			if ( is_null($package['type']) ) {
				$package['type'] = "COMPONENT";
			}
			$msg = JText::sprintf('COM_INSTALLER_MSG_UPDATE_ERROR', JText::_('COM_INSTALLER_TYPE_TYPE_'.strtoupper($package['type'])));
			$result[0][1] = $msg;
			$result[0][0] = false;
			
		} else {
			// Package updated successfully
			if ( is_null($package['type']) ) {
				$package['type'] = "COMPONENT";
			}
			$msg = JText::sprintf('COM_INSTALLER_MSG_UPDATE_SUCCESS', JText::_('COM_INSTALLER_TYPE_TYPE_'.strtoupper($package['type'])));
			$result[0][1] = $msg;
			$result[0][0] = true;
			
		}

		// Quick change
		$this->type = $package['type'];

		// Cleanup the install files
		if (!is_file($package['packagefile'])) {
			$config = JFactory::getConfig();
			$package['packagefile'] = $config->get('tmp_path') . '/' . $package['packagefile'];
		}

		JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

		return $result;
	}
	
	// Funcin que obtiene informacin del sistema (extrada del core)
	private function getInfo()
	{
		if (is_null($this->info))
		{
			$this->info = array();
			$version = new JVersion;
			$platform = new JPlatform;
			$db = JFactory::getDbo();

			if (isset($_SERVER['SERVER_SOFTWARE']))
			{
				$sf = $_SERVER['SERVER_SOFTWARE'];
			}
			else
			{
				$sf = getenv('SERVER_SOFTWARE');
			}

			$this->info['php']			= php_uname();
			$this->info['dbversion']	= $db->getVersion();
			$this->info['dbcollation']	= $db->getCollation();
			$this->info['phpversion']	= phpversion();
			$this->info['server']		= $sf;
			$this->info['sapi_name']	= php_sapi_name();
			$this->info['version']		= $version->getLongVersion();
			$this->info['platform']		= $platform->getLongVersion();
			$this->info['useragent']	= isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "";
		}
	}
	
	// Funcin que devuelve informacin sobre las extensiones no actualizadas
	private function getNotUpdatedExtensions(){
		
		// Habilitamos los sitios deshabilitados
		$enable = $this->enableSites();
		
		// Purgamos la cach y lanzamos la tarea
		$find = $this->findUpdates();
		
		$db = JFactory::getDBO();
		
		// Extraemos el la informacin de las extensiones que necesitan ser actualizadas, que se caracterizan porque su extension_id es distinto a cero
		$query = 'SELECT update_id,extension_id,name,type,version FROM #__updates WHERE extension_id != 0';
		$db->setQuery( $query );
		$extensions = $db->loadRowList();
						
		// Devolvemos el resultado en formato JSON
		return json_encode($extensions);
		
	
	}
	
	/**
	 * Finds updates for an extension.
	 *
	 * @param   int  $eid            Extension identifier to look for
	 * @param   int  $cache_timeout  Cache timout
	 *
	 * @return  boolean Result
	 *
	 * @since   1.6
	 *
	 * Original en /administrator/components/com_installer/models/update.php
	 */
	public function findUpdates($eid = 0, $cache_timeout = 0)
	{
		// Purge the updates list
		$this->purge();

		$updater = JUpdater::getInstance();
		$updater->findUpdates($eid, $cache_timeout);
		return true;
	}
	
	/**
	 * Removes all of the updates from the table.
	 *
	 * @return  boolean result of operation
	 *
	 * @since   1.6
	 *
	 * Original en /administrator/components/com_installer/models/update.php
	 */
	public function purge()
	{
		$db = JFactory::getDbo();

		// Note: TRUNCATE is a DDL operation
		// This may or may not mean depending on your database
		$db->setQuery('TRUNCATE TABLE #__updates');
		if ($db->execute())
		{
			// Reset the last update check timestamp
			$query = $db->getQuery(true)
				->update($db->quoteName('#__update_sites'))
				->set($db->quoteName('last_check_timestamp') . ' = ' . $db->quote(0));
			$db->setQuery($query);
			$db->execute();
			
		}		
	}
	
	/**
	 * Enables any disabled rows in #__update_sites table
	 *
	 * @return  boolean result of operation
	 *
	 * @since   1.6
	 *
	 * Original en /administrator/components/com_installer/models/update.php
	 */
	public function enableSites()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->update('#__update_sites')
			->set('enabled = 1')
			->where('enabled = 0');
		$db->setQuery($query);
		$db->execute();		
	}
	
	/* Funcin que busca si una extensin pasada como argumento utiliza el mecanismo de actualizacin de Akeeba LiveUpdate */
	private function LookForPro($extension_id) {
	
		// Inicializamos las variables
		$found = array();
		$db = JFactory::getDBO();
		
		// Extraemos el nombre de la extensin...
		$query = "SELECT element FROM #__updates WHERE extension_id={$extension_id}";
		$db->setQuery( $query );
		$db->execute();
		$extension_name = $db->loadResult();
		
		// ... y miramos si existen los archivos usados por AKeeba Live Update	
		if ( (JFile::exists(JPATH_ROOT. DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . $extension_name . DIRECTORY_SEPARATOR . 'liveupdate' . DIRECTORY_SEPARATOR . 'liveupdate.php')) && (JFile::exists(JPATH_ROOT. DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . $extension_name . DIRECTORY_SEPARATOR . 'liveupdate' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'model.php')) ) {
			$found[0][1] = $this->Update_LiveUpdate($extension_name);
			$found[0][0] = true;
		}
		
		return $found;
	
	}
	
	/* Funcin que actualiza una extensin pasada como argumento que usa el mecanismo de Akeeba LiveUpdate */
	private function Update_LiveUpdate($extension_name)
	{
	// Inicializamos las variables
		$message = JText::_('COM_SECURITYCHECKPRO_UPDATE_SUCCESSFUL');;
		// Download 
		require_once JPATH_ROOT. DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . $extension_name . DIRECTORY_SEPARATOR . 'liveupdate' . DIRECTORY_SEPARATOR . 'liveupdate.php';
		require_once JPATH_ROOT. DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR . $extension_name . DIRECTORY_SEPARATOR . 'liveupdate' . DIRECTORY_SEPARATOR . 'classes' . DIRECTORY_SEPARATOR . 'model.php';
		
		// Do we need to update?
		$updateInformation = LiveUpdate::getUpdateInformation();
		if(!$updateInformation->hasUpdates) {
			$message = JText::_('COM_SECURITYCHECKPRO_NO_UPDATES_AVAILABLE');
			return $message;
		}

		$model = new LiveupdateModel();
		$ret = $model->download();

		$session = JFactory::getSession();
		$target		= $session->get('target', '', 'liveupdate');
		$tempdir	= $session->get('tempdir', '', 'liveupdate');

		if(!$ret) {
			// An error ocurred :(
			$this->data = 'Could not download the update package';
			$this->status = self::STATUS_ERROR;
			$this->cipher = self::CIPHER_RAW;
			$message = $this->data;			
		} else {
			// Extract
			$ret = $model->extract();

			JLoader::import('joomla.filesystem.file');
			JFile::delete($target);
			
			if(!$ret) {
				// An error ocurred :(
				$this->data = 'Could not extract the update package';
				$this->status = self::STATUS_ERROR;
				$this->cipher = self::CIPHER_RAW;
				$message = $this->data;	
			} else {
				// Install
				$ret = $model->install();

				if(!$ret) {
					// An error ocurred :(
					$this->data = 'Could not install the update package';
					$this->status = self::STATUS_ERROR;
					$this->cipher = self::CIPHER_RAW;
					$message = $this->data;						
				} else {
					// Update cleanup
					$ret = $model->cleanup();

					JLoader::import('joomla.filesystem.file');
					JFile::delete($target);					
					
				}
			}
		}
	}
	
	/* Funcin que actualiza todas las extensiones desactualizadas */
	private function UpdateAll() {
	
		$db = JFactory::getDBO();
		
		// Extraemos el la informacin de las extensiones que necesitan ser actualizadas, que se caracterizan porque su extension_id es distinto a cero
		$query = 'SELECT extension_id FROM #__updates WHERE extension_id != 0';
		$db->setQuery( $query );
		$extension_id_array = $db->loadRowList();
		
		/* Inicializamos la variable 'data', que contendrn las extensiones a actualizar. Tendrn el formato {"0":"1","1":"43"}, donde el primer nmero indica
		el elemento del array y el segundo el id de la extensin */
		$data = array();
		/* Esta variable contendr el ndice del array de extensiones; si no lo usamos, el array json que devolvemos tendra la sintaxis {"0":"1","0":"43"} en lugar de {"0":"1","1":"43"}, por lo que todos los ndices del array sera el 0 */
		$indice = 0;
		
		foreach( $extension_id_array as $extension_id ){ 			
			foreach( $extension_id as $key => $value ){ 
				// Extraemos el par key:value y lo almacenamos en el array
				$key = '"' . addslashes($key+$indice) . '"';
				$value = '"' . addslashes($value) . '"';
				$data[] = $key . ":" . $value; 
				$indice++;
			}
		}
		
		// Extraemos los datos del array y los codificamos en formato JSON
        $data_json = "{" . implode( ",", $data ) . "}";
		
		// A continuacin, llamamos al mtodo UpdateExtension y le pasamos el array en formato json obtenido. ste se encargar de todo el proceso.
		$this->UpdateExtension($data_json,true);		
		
	}
	
	/* Funcin que actualiza un array de extensiones (en formato json) pasado como argumento */
	private function UpdateExtension($extension_id_array,$updateall = false) {
	
		// Inicializamos las variables
		$array_result = array();
		$db = JFactory::getDBO();
		jimport( 'joomla.updater.update' );
		
		// Tenemos que actualizar todas las extensiones. En este caso recibimos un string en lugar de un array (en formato json).
		if ( $updateall ) {
			$extension_id_array = json_decode($extension_id_array,true);			
		}
		
		// Para cada extensin, realizamos su actualizacin
		foreach($extension_id_array as $extension_id) {
		
			if ( $extension_id == 700 ) { // Si el id de la extensin es el 700, se trata del core de Joomla. Lo tratamos de forma diferente.
				$result_core = $this->UpdateCore();
				array_push($array_result,array('Core',$result_core));
			} else if ( $extension_id == 0 ) { // Si el id de la extensin es el 0, significa que queremos actualizar todas las extensiones.
				$this->UpdateAll();
				// Devolvemos el array que contiene los resultados
				return $this->data;
			} else {			
				// Extraemos el nombre de la extensin para mostrarlo en los resultados
				$query = "SELECT name FROM #__updates WHERE extension_id={$extension_id}";
				$db->setQuery( $query );
				$db->execute();
				$extension_name = $db->loadResult();
				// Extraemos la url de la extensin, que contendr la informacin de actualizacin
				$query = "SELECT detailsurl FROM #__updates WHERE extension_id={$extension_id}";
				$db->setQuery( $query );
				$db->execute();
				$detailsurl = $db->loadResult();
				
				// Instanciamos el objeto JUpdate y cargamos los detalles de la actualizacin
				$update = new JUpdate();
				$update->loadFromXML($detailsurl);
				
				// Le pasamos a la funcin de actualizacin el objeto con los detalles de la actualizacin
				$result= $this->install_update($update);
				if ( !$result[0][0] ) {  // Se ha producido un error, es una versin que utiliza algn mecanismo de pago?
					$result_pro = $this->LookForPro($extension_id);
					array_push($array_result,array($extension_name,$result_pro));						
				} else {
					// Guardamos el id de la extensin junto con el resultado
					array_push($array_result,array($extension_name,$result));					
				}				
			}
						
		}	
		
		// Devolvemos el resultado
		$this->data = array(
			'update_result'		=> $array_result
		);	
		
	}
	
	/* Funcin que realiza una copia de seguridad usando Akeeba y su funcin de copias de seguridad va frontend. La clave usada se pasa como argumento */
	private function Backup($data) {
		
		
		// URI del sitio
		$uri = JURI::root();
		
		// Desencriptamos los datos recibidos, que vendrn como un array (vase data[0]) y en formato json
		$response = $this->decrypt($data[0], $this->password);
		$response = json_decode($response,true);	
		
		// Extraemos la clave pblica de Akeeba, que vendr en el elemento 'akeeba_key' del array
		$akeeba_key = $response['frontend_key'];
		
		// Inicializamos la tarea
		$ch = curl_init($uri . "?option=com_akeeba&view=backup&key=" . $akeeba_key);
			
		// Configuracin extrada de https://www.akeebabackup.com/documentation/akeeba-backup-documentation/automating-your-backup.html
		curl_setopt($ch, CURLOPT_HEADER, false);  // Este valor es false para que no incluya en la respuesta la cabecera HTTP
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION,TRUE);
		curl_setopt($ch, CURLOPT_MAXREDIRS,10000); # Fix by Nicholas
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
					
		$response = curl_exec($ch);
		curl_close($ch);
		
		// Devolvemos el resultado
		$this->data = array(
			'backup'		=> $response
		);
	}	
	
	/* Funcin que instala una extensin desde una url. La ur se pasa como argumento */
	private function Upload_install($data) {
		
		// Inicialiamos las variables
		$result = true;
		
		/* Cargamos el lenguaje del componente 'com_installer' */
		$lang = JFactory::getLanguage();
		$lang->load('com_installer',JPATH_ADMINISTRATOR);
				
		// Desencriptamos los datos recibidos, que vendrn como un array (vase data[0]) y en formato json
		$response = $this->decrypt($data[0], $this->password);
		$response = json_decode($response,true);	
		
		// Url del paquete a instalar
		$url = $response['path_to_file'];
		
		$package = null;
		
		// Extraemos el paquete desde la url pasada
		$package = $this->getPackageFromUrl($url);
		
		// Was the package unpacked?
		if (!$package || !$package['type'])
		{
			if (in_array($installType, array('upload', 'url')))
			{
				JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);
			}

			$msg = JText::_('COM_INSTALLER_UNABLE_TO_FIND_INSTALL_PACKAGE');
			return false;
		}

		// Get an installer instance
		$installer = JInstaller::getInstance();

		// Install the package
		if (!$installer->install($package['dir']))
		{
			// There was an error installing the package
			$msg = JText::sprintf('COM_INSTALLER_INSTALL_ERROR', JText::_('COM_INSTALLER_TYPE_TYPE_' . strtoupper($package['type'])));
			$result = false;
		}
		else
		{
			// Package installed sucessfully
			$msg = JText::sprintf('COM_INSTALLER_INSTALL_SUCCESS', JText::_('COM_INSTALLER_TYPE_TYPE_' . strtoupper($package['type'])));			
		}

		// Cleanup the install files
		if (!is_file($package['packagefile']))
		{
			$config = JFactory::getConfig();
			$package['packagefile'] = $config->get('tmp_path') . '/' . $package['packagefile'];
		}

		$cleanup_resume = JInstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);
		
		// Si el borrado de los ficheros de instalacin falla, lo hacemos 'artesanalmente'
		if ( !$cleanup_resume ) {
			$config = JFactory::getConfig();
			$root = $config->get('tmp_path');
			$files_name = JFolder::files($root,'.',true,true);
			foreach ( $files_name as $file ) {
				JFile::delete($root . DIRECTORY_SEPARATOR . $file);
			}
		}
		
		// Recogemos los mensajes encolados para mostrar ms informacin
		$enqueued_messages = JFactory::getApplication()->getMessageQueue();
		
		// Devolvemos el resultado
		$this->data = array(
			'upload_install'		=> $result,
			'message'	=>	$msg,
			'enqueued_messages'	=> $enqueued_messages
		);
	}
	
	/**
	 * Install an extension from a URL
	 *
	 * @return  Package details or false on failure
	 *
	 * @since   1.5
	 */
	protected function getPackageFromUrl($url)
	{
		
		// Did you give us a URL?
		if (!$url)
		{
			JError::raiseWarning('', JText::_('COM_INSTALLER_MSG_INSTALL_ENTER_A_URL'));
			return false;
		}

		// Handle updater XML file case:
		if (preg_match('/\.xml\s*$/', $url))
		{
			jimport('joomla.updater.update');
			$update = new JUpdate;
			$update->loadFromXML($url);
			$package_url = trim($update->get('downloadurl', false)->_data);
			if ($package_url)
			{
				$url = $package_url;
			}
			unset($update);
		}

		// Download the package at the URL given
		$p_file = JInstallerHelper::downloadPackage($url);

		// Was the package downloaded?
		if (!$p_file)
		{
			JError::raiseWarning('', JText::_('COM_INSTALLER_MSG_INSTALL_INVALID_URL'));
			return false;
		}

		$config   = JFactory::getConfig();
		$tmp_dest = $config->get('tmp_path');

		// Unpack the downloaded package file
		$package = JInstallerHelper::unpack($tmp_dest . '/' . $p_file, true);

		return $package;
	}
	
	/**
	 * Downloads the update package to the site.
	 *
	 * @return  bool|string False on failure, basename of the file in any other case.
	 *
	 * @since   2.5.4
	 */
	public function download_core($packageURL)
	{
		$basename = basename($packageURL);

		// Find the path to the temp directory and the local package.
		$config = JFactory::getConfig();
		$tempdir = $config->get('tmp_path');
		$target = $tempdir . '/' . $basename;

		// Do we have a cached file?
		$exists = JFile::exists($target);

		if (!$exists)
		{
			// Not there, let's fetch it.
			return $this->downloadPackage($packageURL, $target);
		}
		else
		{
			// Is it a 0-byte file? If so, re-download please.
			$filesize = @filesize($target);

			if (empty($filesize))
			{
				return $this->downloadPackage($packageURL, $target);
			}

			// Yes, it's there, skip downloading.
			return $basename;
		}
	}

	/**
	 * Downloads a package file to a specific directory
	 *
	 * @param   string  $url     The URL to download from
	 * @param   string  $target  The directory to store the file
	 *
	 * @return  boolean True on success
	 *
	 * @since   2.5.4
	 */
	protected function downloadPackage($url, $target)
	{
		// Get the handler to download the package
		try
		{
			$http = JHttpFactory::getHttp(null, array('curl', 'stream'));
		}
		catch (RuntimeException $e)
		{
			return false;
		}

		jimport('joomla.filesystem.file');

		// Make sure the target does not exist.
		JFile::delete($target);

		// Download the package
		$result = $http->get($url);

		if (!$result || ($result->code != 200 && $result->code != 310))
		{
			return false;
		}

		// Write the file to disk
		JFile::write($target, $result->body);

		return basename($target);
	}
	
	/**
	 * Create restoration file.
	 *
	 * @param   string  $basename  Optional base path to the file.
	 *
	 * @return  boolean True if successful; false otherwise.
	 *
	 * @since  2.5.4
	 */
	public function createRestorationFile($basename = null)
	{
		// Get a password
		$password = JUserHelper::genRandomPassword(32);
		$app = JFactory::getApplication();
		$app->setUserState('com_joomlaupdate.password', $password);

		// Do we have to use FTP?
		$method = $app->input->get('method', 'direct');

		// Get the absolute path to site's root.
		$siteroot = JPATH_SITE;

		// If the package name is not specified, get it from the update info.
		if (empty($basename))
		{
			$updateInfo = $this->getUpdateInformation();
			$packageURL = $updateInfo['object']->downloadurl->_data;
			$basename = basename($packageURL);
		}

		// Get the package name.
		$config = JFactory::getConfig();
		$tempdir = $config->get('tmp_path');
		$file = $tempdir . '/' . $basename;

		$filesize = @filesize($file);
		$app->setUserState('com_joomlaupdate.password', $password);
		$app->setUserState('com_joomlaupdate.filesize', $filesize);

		$data = "<?php\ndefined('_AKEEBA_RESTORATION') or die('Restricted access');\n";
		$data .= '$restoration_setup = array(' . "\n";
		$data .= <<<ENDDATA
	'kickstart.security.password' => '$password',
	'kickstart.tuning.max_exec_time' => '5',
	'kickstart.tuning.run_time_bias' => '75',
	'kickstart.tuning.min_exec_time' => '0',
	'kickstart.procengine' => '$method',
	'kickstart.setup.sourcefile' => '$file',
	'kickstart.setup.destdir' => '$siteroot',
	'kickstart.setup.restoreperms' => '0',
	'kickstart.setup.filetype' => 'zip',
	'kickstart.setup.dryrun' => '0'
ENDDATA;

		if ($method == 'ftp')
		{
			/*
			 * Fetch the FTP parameters from the request. Note: The password should be
			 * allowed as raw mode, otherwise something like !@<sdf34>43H% would be
			 * sanitised to !@43H% which is just plain wrong.
			 */
			$ftp_host = $app->input->get('ftp_host', '');
			$ftp_port = $app->input->get('ftp_port', '21');
			$ftp_user = $app->input->get('ftp_user', '');
			$ftp_pass = $app->input->get('ftp_pass', '', 'default', 'none', 2);
			$ftp_root = $app->input->get('ftp_root', '');

			// Is the tempdir really writable?
			$writable = @is_writeable($tempdir);

			if ($writable)
			{
				// Let's be REALLY sure.
				$fp = @fopen($tempdir . '/test.txt', 'w');

				if ($fp === false)
				{
					$writable = false;
				}
				else
				{
					fclose($fp);
					unlink($tempdir . '/test.txt');
				}
			}

			// If the tempdir is not writable, create a new writable subdirectory.
			if (!$writable)
			{
				$FTPOptions = JClientHelper::getCredentials('ftp');
				$ftp = JClientFtp::getInstance($FTPOptions['host'], $FTPOptions['port'], null, $FTPOptions['user'], $FTPOptions['pass']);
				$dest = JPath::clean(str_replace(JPATH_ROOT, $FTPOptions['root'], $tempdir . '/admintools'), '/');

				if (!@mkdir($tempdir . '/admintools'))
				{
					$ftp->mkdir($dest);
				}

				if (!@chmod($tempdir . '/admintools', 511))
				{
					$ftp->chmod($dest, 511);
				}

				$tempdir .= '/admintools';
			}

			// Just in case the temp-directory was off-root, try using the default tmp directory.
			$writable = @is_writeable($tempdir);

			if (!$writable)
			{
				$tempdir = JPATH_ROOT . '/tmp';

				// Does the JPATH_ROOT/tmp directory exist?
				if (!is_dir($tempdir))
				{
					JFolder::create($tempdir, 511);
					JFile::write($tempdir . '/.htaccess', "order deny,allow\ndeny from all\nallow from none\n");
				}

				// If it exists and it is unwritable, try creating a writable admintools subdirectory.
				if (!is_writable($tempdir))
				{
					$FTPOptions = JClientHelper::getCredentials('ftp');
					$ftp = JClientFtp::getInstance($FTPOptions['host'], $FTPOptions['port'], null, $FTPOptions['user'], $FTPOptions['pass']);
					$dest = JPath::clean(str_replace(JPATH_ROOT, $FTPOptions['root'], $tempdir . '/admintools'), '/');

					if (!@mkdir($tempdir . '/admintools'))
					{
						$ftp->mkdir($dest);
					}

					if (!@chmod($tempdir . '/admintools', 511))
					{
						$ftp->chmod($dest, 511);
					}

					$tempdir .= '/admintools';
				}
			}

			// If we still have no writable directory, we'll try /tmp and the system's temp-directory.
			$writable = @is_writeable($tempdir);

			if (!$writable)
			{
				if (@is_dir('/tmp') && @is_writable('/tmp'))
				{
					$tempdir = '/tmp';
				}
				else
				{
					// Try to find the system temp path.
					$tmpfile = @tempnam("dummy", "");
					$systemp = @dirname($tmpfile);
					@unlink($tmpfile);

					if (!empty($systemp))
					{
						if (@is_dir($systemp) && @is_writable($systemp))
						{
							$tempdir = $systemp;
						}
					}
				}
			}

			$data .= <<<ENDDATA
	,
	'kickstart.ftp.ssl' => '0',
	'kickstart.ftp.passive' => '1',
	'kickstart.ftp.host' => '$ftp_host',
	'kickstart.ftp.port' => '$ftp_port',
	'kickstart.ftp.user' => '$ftp_user',
	'kickstart.ftp.pass' => '$ftp_pass',
	'kickstart.ftp.dir' => '$ftp_root',
	'kickstart.ftp.tempdir' => '$tempdir'
ENDDATA;
		}

		$data .= ');';

		// Remove the old file, if it's there...
		$configpath = JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'administrator' . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR .'com_joomlaupdate' . DIRECTORY_SEPARATOR . 'restoration.php';

		if (JFile::exists($configpath))
		{
			JFile::delete($configpath);
		}

		// Write new file. First try with JFile.
		$result = JFile::write($configpath, $data);

		// In case JFile used FTP but direct access could help.
		if (!$result)
		{
			if (function_exists('file_put_contents'))
			{
				$result = @file_put_contents($configpath, $data);

				if ($result !== false)
				{
					$result = true;
				}
			}
			else
			{
				$fp = @fopen($configpath, 'wt');

				if ($fp !== false)
				{
					$result = @fwrite($fp, $data);

					if ($result !== false)
					{
						$result = true;
					}

					@fclose($fp);
				}
			}
		}

		return $result;
	}	
	
	/**
	 * Runs the schema update SQL files, the PHP update script and updates the
	 * manifest cache and #__extensions entry. Essentially, it is identical to
	 * JInstallerFile::install() without the file copy.
	 *
	 * @return  boolean True on success.
	 *
	 * @since   2.5.4
	 */
	public function finaliseUpgrade()
	{
		$installer = JInstaller::getInstance();

		$manifest = $installer->isManifest(JPATH_MANIFESTS . '/files/joomla.xml');

		if ($manifest === false)
		{
			$installer->abort(JText::_('JLIB_INSTALLER_ABORT_DETECTMANIFEST'));

			return false;
		}

		$installer->manifest = $manifest;

		$installer->setUpgrade(true);
		$installer->setOverwrite(true);

		$installer->extension = JTable::getInstance('extension');
		$installer->extension->load(700);

		$installer->setAdapter($installer->extension->type);

		$installer->setPath('manifest', JPATH_MANIFESTS . '/files/joomla.xml');
		$installer->setPath('source', JPATH_MANIFESTS . '/files');
		$installer->setPath('extension_root', JPATH_ROOT);

		$manifestPath = JPath::clean($installer->getPath('manifest'));
		
		// Run the script file.
		JLoader::register('JoomlaInstallerScript', JPATH_ADMINISTRATOR . '/components/com_admin/script.php');

		$manifestClass = new JoomlaInstallerScript;

		ob_start();
		ob_implicit_flush(false);

		if ($manifestClass && method_exists($manifestClass, 'preflight'))
		{
			if ($manifestClass->preflight('update', $installer) === false)
			{
				$installer->abort(JText::_('JLIB_INSTALLER_ABORT_FILE_INSTALL_CUSTOM_INSTALL_FAILURE'));

				return false;
			}
		}

		// Create msg object; first use here.
		$msg = ob_get_contents();
		ob_end_clean();

		// Get a database connector object.
		$db = $this->getDbo();

		/*
		 * Check to see if a file extension by the same name is already installed.
		 * If it is, then update the table because if the files aren't there
		 * we can assume that it was (badly) uninstalled.
		 * If it isn't, add an entry to extensions.
		 */
		$query = $db->getQuery(true)
			->select($db->quoteName('extension_id'))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('type') . ' = ' . $db->quote('file'))
			->where($db->quoteName('element') . ' = ' . $db->quote('joomla'));
		$db->setQuery($query);

		try
		{
			$db->execute();
		}
		catch (RuntimeException $e)
		{
			// Install failed, roll back changes.
			$installer->abort(
				JText::sprintf('JLIB_INSTALLER_ABORT_FILE_ROLLBACK', JText::_('JLIB_INSTALLER_UPDATE'), $e->getMessage())
			);

			return false;
		}

		$id = $db->loadResult();
		$row = JTable::getInstance('extension');

		if ($id)
		{
			// Load the entry and update the manifest_cache.
			$row->load($id);

			// Update name.
			$row->set('name', 'files_joomla');

			// Update manifest.
			$row->manifest_cache = $installer->generateManifestCache();

			if (!$row->store())
			{
				// Install failed, roll back changes.
				$installer->abort(
					JText::sprintf('JLIB_INSTALLER_ABORT_FILE_ROLLBACK', JText::_('JLIB_INSTALLER_UPDATE'), $row->getError())
				);

				return false;
			}
		}
		else
		{
			// Add an entry to the extension table with a whole heap of defaults.
			$row->set('name', 'files_joomla');
			$row->set('type', 'file');
			$row->set('element', 'joomla');

			// There is no folder for files so leave it blank.
			$row->set('folder', '');
			$row->set('enabled', 1);
			$row->set('protected', 0);
			$row->set('access', 0);
			$row->set('client_id', 0);
			$row->set('params', '');
			$row->set('system_data', '');
			$row->set('manifest_cache', $installer->generateManifestCache());

			if (!$row->store())
			{
				// Install failed, roll back changes.
				$installer->abort(JText::sprintf('JLIB_INSTALLER_ABORT_FILE_INSTALL_ROLLBACK', $row->getError()));

				return false;
			}

			// Set the insert id.
			$row->set('extension_id', $db->insertid());

			// Since we have created a module item, we add it to the installation step stack
			// so that if we have to rollback the changes we can undo it.
			$installer->pushStep(array('type' => 'extension', 'extension_id' => $row->extension_id));
		}

		$result = $installer->parseSchemaUpdates($manifest->update->schemas, $row->extension_id);

		if ($result === false)
		{
			// Install failed, rollback changes.
			$installer->abort(JText::sprintf('JLIB_INSTALLER_ABORT_FILE_UPDATE_SQL_ERROR', $db->stderr(true)));

			return false;
		}

		// Start Joomla! 1.6.
		ob_start();
		ob_implicit_flush(false);

		if ($manifestClass && method_exists($manifestClass, 'update'))
		{
			if ($manifestClass->update($installer) === false)
			{
				// Install failed, rollback changes.
				$installer->abort(JText::_('JLIB_INSTALLER_ABORT_FILE_INSTALL_CUSTOM_INSTALL_FAILURE'));

				return false;
			}
		}

		// Append messages.
		$msg .= ob_get_contents();
		ob_end_clean();

		// Clobber any possible pending updates.
		$update = JTable::getInstance('update');
		$uid = $update->find(
			array('element' => 'joomla', 'type' => 'file', 'client_id' => '0', 'folder' => '')
		);

		if ($uid)
		{
			$update->delete($uid);
		}

		// And now we run the postflight.
		ob_start();
		ob_implicit_flush(false);

		if ($manifestClass && method_exists($manifestClass, 'postflight'))
		{
			$manifestClass->postflight('update', $installer);
		}

		// Append messages.
		$msg .= ob_get_contents();
		ob_end_clean();

		if ($msg != '')
		{
			$installer->set('extension_message', $msg);
		}

		// Refresh versionable assets cache.
		JFactory::getApplication()->flushAssets();

		return true;
	}
	
}